local _, AT = ...

-- file import
local Utils = AT.Utils
local Settings = AT.Settings
local Strings = AT.Strings

-- enum imports
local PROFILE_TYPE = AT.Enums.PROFILE_TYPE

-- constants
local CAMEL_CASE_MAP = {
  loadnext = "LoadNext"
}

-- tokenizer
local Tokenizer = {
  tokens = {},
  index = 1,
  Parse = function(self, aStr)
    wipe(self.tokens)
    self.index = 1
    for w in aStr:gmatch("%S+") do 
      tinsert(self.tokens, strlower(w))
    end
  end,
  Current = function(self)
    return self.tokens[self.index]
  end,
  Next = function(self)
    self.index = self.index + 1
    return self:Current()
  end
}

-- profile utils
local function GetProfileType(aStr)
  return aStr ~= nil and rawget(PROFILE_TYPE, strupper(aStr)) or nil
end

local function GetProfileFunc(aStr)
  local funcOwner
  -- get function name
  local camelCase = CAMEL_CASE_MAP[aStr]
  if camelCase == nil then
    camelCase = Utils:StrFirstCharToUpper(aStr)
  end
  -- get func from Settings
  funcOwner = Settings
  local func = rawget(funcOwner, camelCase..'Profile')
  if func == nil then
    func = rawget(funcOwner, camelCase..'Profiles')
  end
  -- get func from FrameSettings
  if func == nil then
    funcOwner = AT.FrameSettings
    func = rawget(funcOwner, 'Profiles'..camelCase)
  end
  -- finalize
  if type(func) == 'function' then
    return function(arg1, arg2, arg3, arg4)
      func(funcOwner, arg1, arg2, arg3, arg4)
    end
  end
  return nil
end

-- main logic
SLASH_SPELLSTEALMASTER1 = "/spellstealmaster"
SlashCmdList['SPELLSTEALMASTER'] = function(aMsg)
  -- no args, show options frame
  if Utils:StrIsNilOrEmpty(aMsg) then
    -- called twice because when interface settings where not opened before the category gets not selected
    InterfaceOptionsFrame_OpenToCategory(AT.FrameSettings.categoryName)
    InterfaceOptionsFrame_OpenToCategory(AT.FrameSettings.categoryName)
    return
  end

  -- split msg into tokens
  Tokenizer:Parse(aMsg)

  local token = Tokenizer:Current()

  -- early out on no tokens
  if token == nil then
    return
  end

  -- profile commands
  if token == 'p' or token == 'profile' or token == 'profiles' then
    -- get func
    local cmd = Tokenizer:Next()
    if cmd ~= nil then
      local cmdFunc = GetProfileFunc(cmd)
      if cmdFunc then
        -- get args
        local pt = GetProfileType(Tokenizer:Next())
        local name1 = Tokenizer:Next()
        local name2 = Tokenizer:Next()
        if ((pt == nil and name1 == nil and name2 == nil) and (cmd == 'help' or cmd == 'list'))
          or ((pt ~= nil and name1 == nil and name2 == nil) and (cmd == 'loadnext' or cmd == 'list'))
          or ((pt ~= nil and name1 ~= nil and name2 == nil) and (cmd == 'create' or cmd == 'load' or cmd == 'delete'))
          or ((pt ~= nil and name1 ~= nil and name2 ~= nil) and (cmd == 'copy' or cmd == 'rename')) then
          -- call command
          cmdFunc(pt, name1, name2)
          return
        end
      end
      -- print wrong profile command usage
      local cmdUsage = Strings.profileCommands[cmd]
      if cmd ~= nil then
        Utils:Print(string.format(Strings.profileCmdFailed, cmdUsage))
        return
      end
    end
    -- print invalid profile command
    Utils:Print(Strings.profileCmdInvalid)
    return
  end

  -- print invalid command
  Utils:Print("invalid command")
end
