local _, AT = ...

-- file import
local Settings = AT.Settings
local Utils = AT.Utils

-- emum import
local BUFF_INFO = AT.Enums.BUFF_INFO
local TRANSPARENCY_STATE = AT.Enums.TRANSPARENCY_STATE

-- consts
local FRAME_TARGET_WIDTH = 155
local FRAME_TARGET_HEIGHT = 45

-- "class" setup
local FrameImportant = {}
setmetatable(FrameImportant, {__index = AT.FrameBase})
FrameImportant.frame = nil
FrameImportant.dragable = true
FrameImportant.padding = 10
FrameImportant.allowedTransparencyStates = {
  TRANSPARENCY_STATE.NOT_USABLE,
  TRANSPARENCY_STATE.NOT_IN_RANGE
}

AT.FrameImportant = FrameImportant

function FrameImportant:construct(aId, aHeadline)
  obj = {}
  setmetatable(obj, {__index = self})
  obj:postConstruct(aId)
  obj:Init(aHeadline)
  return obj
end

function FrameImportant:Init(aHeadline)
  -- create frame
  self.frame = CreateFrame('FRAME', nil, UIParent)

  -- enable mouse input for spell tooltips
  self.frame:EnableMouse(true)
  self.frame:SetHyperlinksEnabled(true)
  self.frame:SetScript('OnHyperlinkClick', ChatFrame_OnHyperlinkShow)

  -- add headline
  self.frame.headline = self.frame:CreateFontString(nil, 'OVERLAY', GameFontNormal)
  self.frame.headline:SetPoint('TOP', 0, -10)
  self.frame.headline:SetJustifyV('TOP')
  self.frame.headline:SetJustifyH('CENTER')
  self.frame.headline:SetFont('Fonts\\FRIZQT__.TTF', 18, 'OUTLINE')
  self.frame.headline:SetNonSpaceWrap(false)
  self.frame.headline:SetText(aHeadline)

  -- add stealable spells list
  self.frame.text = self.frame:CreateFontString(nil, 'OVERLAY', GameFontNormal)
  self.frame.text:SetPoint('CENTER', 0, -10)
  self.frame.text:SetJustifyV('TOP')
  self.frame.text:SetJustifyH('CENTER')
  self.frame.text:SetFont('Fonts\\FRIZQT__.TTF', 12, 'OUTLINE')
  self.frame.text:SetNonSpaceWrap(false)
  self.frame.text:SetText('')

  self:PostInit()
  self:Hide()
end

-- @OVERRIDE
function FrameImportant:UpdateSettings()
  -- update headline color
  local c = Settings.colorHeadline
  self.frame.headline:SetTextColor(c.r, c.g, c.b, c.a)

  -- base call
  AT.FrameBase.UpdateSettings(self)
end

function FrameImportant:Hide()
  self.isVisible = false
  self.frame:Hide()
end

function FrameImportant:Set(aTargetUnit, aStealableBuffInfo) 
  if not self:GetEnabled() then
    return
  end

  local text = ''

  for _, buffInfo in ipairs(aStealableBuffInfo) do
    -- add main string
    local emptyOrLinebreak = (text == '') and '' or '\n'
    local emptyOrCount = (buffInfo[BUFF_INFO.COUNT] > 1) and '|c'..self.textColors.buffCount..buffInfo[BUFF_INFO.COUNT]..'x' or ''
    text = string.format('%s%s|Hspell:%s|h%s|T%s:14|t|c%s %s|h',
      text, emptyOrLinebreak, buffInfo[BUFF_INFO.ID], emptyOrCount, buffInfo[BUFF_INFO.ICON], self.textColors.buffName, buffInfo[BUFF_INFO.NAME])
    -- add experation time
    if buffInfo[BUFF_INFO.EXPIRATION_TIME] ~= nil and buffInfo[BUFF_INFO.EXPIRATION_TIME] > 0 then
      text = string.format('%s|c%s - %s|r', text, self.textColors.buffExpTime, Utils:FormatExpirationTime(buffInfo[BUFF_INFO.EXPIRATION_TIME]))
    end
  end

  self.frame.text:SetText(text)
  self.frame:SetSize(math.max(self.frame.text:GetStringWidth(), self.frame.headline:GetStringWidth(), FRAME_TARGET_WIDTH) + (self.padding * 2), FRAME_TARGET_HEIGHT + self.frame.text:GetStringHeight())
  self.isVisible = true
  self.frame:Show()
end
