local _, AT = ...

-- file imports
local NameplateFrames = AT.NameplateFrames
local Settings = AT.Settings

-- table import
local nameplateMap = AT.nameplateMap

-- enum import
local BUFF_INFO = AT.Enums.BUFF_INFO
local SCAN_CATEGORY = AT.Enums.SCAN_CATEGORY
local SCAN_RESULT = AT.Enums.SCAN_RESULT

local TestData = {}
AT.TestData = TestData

TestData.npBuff = {
  [BUFF_INFO.NAME] = nil,
  [BUFF_INFO.ICON] = nil,
  [BUFF_INFO.COUNT] = 0,
  [BUFF_INFO.EXPIRATION_TIME] = 40,
  [BUFF_INFO.ID] = 80353,
  [BUFF_INFO.SHOW_IN_MOUSEOVER] = false,
  [BUFF_INFO.SHOW_IN_OVERVIEW] = false,
  [BUFF_INFO.SHOW_IN_NAMEPLATE] = true,
}

TestData.data = {
  {
    [SCAN_RESULT.UNIT] = "Your target",
    [SCAN_RESULT.UNIT_GUID] = nil,
    [SCAN_RESULT.CATEGORY] = SCAN_CATEGORY.TARGET,
    [SCAN_RESULT.BUFFS_INFO] = {
      {
        [BUFF_INFO.NAME] = nil,
        [BUFF_INFO.ICON] = nil,
        [BUFF_INFO.COUNT] = 0,
        [BUFF_INFO.EXPIRATION_TIME] = 40,
        [BUFF_INFO.ID] = 80353, -- timewarp
        [BUFF_INFO.SHOW_IN_MOUSEOVER] = false,
        [BUFF_INFO.SHOW_IN_OVERVIEW] = true,
        [BUFF_INFO.SHOW_IN_NAMEPLATE] = false,
      }
    },
    [SCAN_RESULT.HAS_IMPORTANT_BUFFS] = true,
    [SCAN_RESULT.HAS_MOUSEOVER_BUFFS] = false,
    [SCAN_RESULT.HAS_OVERVIEW_BUFFS] = nil,
    [SCAN_RESULT.HAS_NAMEPLATE_BUFFS] = false,
  },
  {
    [SCAN_RESULT.UNIT] = "Your focus",
    [SCAN_RESULT.UNIT_GUID] = nil,
    [SCAN_RESULT.CATEGORY] = SCAN_CATEGORY.FOCUS,
    [SCAN_RESULT.BUFFS_INFO] = {
      {
        [BUFF_INFO.NAME] = nil,
        [BUFF_INFO.ICON] = nil,
        [BUFF_INFO.COUNT] = 5,
        [BUFF_INFO.EXPIRATION_TIME] = 40,
        [BUFF_INFO.ID] = 32182, -- heroism
        [BUFF_INFO.SHOW_IN_MOUSEOVER] = false,
        [BUFF_INFO.SHOW_IN_OVERVIEW] = nil,
        [BUFF_INFO.SHOW_IN_NAMEPLATE] = false,
      }
    },
    [SCAN_RESULT.HAS_IMPORTANT_BUFFS] = true,
    [SCAN_RESULT.HAS_MOUSEOVER_BUFFS] = false,
    [SCAN_RESULT.HAS_OVERVIEW_BUFFS] = nil,
    [SCAN_RESULT.HAS_NAMEPLATE_BUFFS] = false,
  },
  {
    [SCAN_RESULT.UNIT] = "Your mouseover target",
    [SCAN_RESULT.UNIT_GUID] = nil,
    [SCAN_RESULT.CATEGORY] = SCAN_CATEGORY.MOUSEOVER,
    [SCAN_RESULT.BUFFS_INFO] = {
      {
        [BUFF_INFO.NAME] = nil,
        [BUFF_INFO.ICON] = nil,
        [BUFF_INFO.COUNT] = 0,
        [BUFF_INFO.EXPIRATION_TIME] = 40,
        [BUFF_INFO.ID] = 80353, -- timewarp
        [BUFF_INFO.SHOW_IN_MOUSEOVER] = true,
        [BUFF_INFO.SHOW_IN_OVERVIEW] = nil,
        [BUFF_INFO.SHOW_IN_NAMEPLATE] = false,
      }
    },
    [SCAN_RESULT.HAS_IMPORTANT_BUFFS] = true,
    [SCAN_RESULT.HAS_MOUSEOVER_BUFFS] = nil,
    [SCAN_RESULT.HAS_OVERVIEW_BUFFS] = nil,
    [SCAN_RESULT.HAS_NAMEPLATE_BUFFS] = false,
  },
  {
    [SCAN_RESULT.UNIT] = "Patchwerk",
    [SCAN_RESULT.UNIT_GUID] = nil,
    [SCAN_RESULT.CATEGORY] = SCAN_CATEGORY.BOSS,
    [SCAN_RESULT.BUFFS_INFO] = {
      {
        [BUFF_INFO.NAME] = nil,
        [BUFF_INFO.ICON] = nil,
        [BUFF_INFO.COUNT] = 0,
        [BUFF_INFO.EXPIRATION_TIME] = 40,
        [BUFF_INFO.ID] = 80353, -- timewarp
        [BUFF_INFO.SHOW_IN_MOUSEOVER] = false,
        [BUFF_INFO.SHOW_IN_OVERVIEW] = nil,
        [BUFF_INFO.SHOW_IN_NAMEPLATE] = false,
      },
      {
        [BUFF_INFO.NAME] = nil,
        [BUFF_INFO.ICON] = nil,
        [BUFF_INFO.COUNT] = 0,
        [BUFF_INFO.EXPIRATION_TIME] = 40,
        [BUFF_INFO.ID] = 2825, -- bloodlust
        [BUFF_INFO.SHOW_IN_MOUSEOVER] = false,
        [BUFF_INFO.SHOW_IN_OVERVIEW] = nil,
        [BUFF_INFO.SHOW_IN_NAMEPLATE] = false,
      },
      {
        [BUFF_INFO.NAME] = nil,
        [BUFF_INFO.ICON] = nil,
        [BUFF_INFO.COUNT] = 0,
        [BUFF_INFO.EXPIRATION_TIME] = 40,
        [BUFF_INFO.ID] = 32182, -- heroism
        [BUFF_INFO.SHOW_IN_MOUSEOVER] = false,
        [BUFF_INFO.SHOW_IN_OVERVIEW] = nil,
        [BUFF_INFO.SHOW_IN_NAMEPLATE] = false,
      }
    },
    [SCAN_RESULT.HAS_IMPORTANT_BUFFS] = false,
    [SCAN_RESULT.HAS_MOUSEOVER_BUFFS] = false,
    [SCAN_RESULT.HAS_OVERVIEW_BUFFS] = nil,
    [SCAN_RESULT.HAS_NAMEPLATE_BUFFS] = false,
  },
  {
    [SCAN_RESULT.UNIT] = "Random dude 1",
    [SCAN_RESULT.UNIT_GUID] = nil,
    [SCAN_RESULT.CATEGORY] = SCAN_CATEGORY.NAMEPLATE,
    [SCAN_RESULT.BUFFS_INFO] = {
      {
        [BUFF_INFO.NAME] = nil,
        [BUFF_INFO.ICON] = nil,
        [BUFF_INFO.COUNT] = 0,
        [BUFF_INFO.EXPIRATION_TIME] = 40,
        [BUFF_INFO.ID] = 80353, -- timewarp
        [BUFF_INFO.SHOW_IN_MOUSEOVER] = false,
        [BUFF_INFO.SHOW_IN_OVERVIEW] = nil,
        [BUFF_INFO.SHOW_IN_NAMEPLATE] = false,
      }
    },
    [SCAN_RESULT.HAS_IMPORTANT_BUFFS] = false,
    [SCAN_RESULT.HAS_MOUSEOVER_BUFFS] = false,
    [SCAN_RESULT.HAS_OVERVIEW_BUFFS] = nil,
    [SCAN_RESULT.HAS_NAMEPLATE_BUFFS] = false,
  },
  {
    [SCAN_RESULT.UNIT] = "Random dude 2",
    [SCAN_RESULT.UNIT_GUID] = nil,
    [SCAN_RESULT.CATEGORY] = SCAN_CATEGORY.NAMEPLATE,
    [SCAN_RESULT.BUFFS_INFO] = {
      {
        [BUFF_INFO.NAME] = nil,
        [BUFF_INFO.ICON] = nil,
        [BUFF_INFO.COUNT] = 0,
        [BUFF_INFO.EXPIRATION_TIME] = 40,
        [BUFF_INFO.ID] = 80353, -- timewarp
        [BUFF_INFO.SHOW_IN_OVERVIEW] = true
      }
    },
    [SCAN_RESULT.HAS_IMPORTANT_BUFFS] = false,
    [SCAN_RESULT.HAS_MOUSEOVER_BUFFS] = false,
    [SCAN_RESULT.HAS_OVERVIEW_BUFFS] = nil,
    [SCAN_RESULT.HAS_NAMEPLATE_BUFFS] = false,
  },
}

-- prepares test data for ingame use
function TestData:Init()
  -- data for frames
  for _, scanResult in ipairs(self.data) do
    scanResult[SCAN_RESULT.HAS_OVERVIEW_BUFFS] = false
    for _, buffInfo in ipairs(scanResult[SCAN_RESULT.BUFFS_INFO]) do
      local name, _, icon, _, _, _, spellId = GetSpellInfo(buffInfo[BUFF_INFO.ID])
      -- filtering here used to filter target and focus for overview frame
      local _, _, isOverviewEntry, _ = AT.Main:FilterContextual(scanResult[SCAN_RESULT.CATEGORY], buffInfo[BUFF_INFO.ID], name, buffInfo[BUFF_INFO.EXPIRATION_TIME])
      buffInfo[BUFF_INFO.NAME] = name
      buffInfo[BUFF_INFO.ICON] = icon
      buffInfo[BUFF_INFO.SHOW_IN_OVERVIEW] = isOverviewEntry
      if isOverviewEntry then
        scanResult[SCAN_RESULT.HAS_OVERVIEW_BUFFS] = true
      end
    end
    if scanResult[SCAN_RESULT.CATEGORY] == SCAN_CATEGORY.BOSS and not Settings.filterOverviewShowBosses then
      scanResult[SCAN_RESULT.HAS_OVERVIEW_BUFFS] = false
    end
    -- show mouseover frame depending on the test frame mouseover setting
    if scanResult[SCAN_RESULT.CATEGORY] == SCAN_CATEGORY.MOUSEOVER then
      scanResult[SCAN_RESULT.HAS_MOUSEOVER_BUFFS] = Settings.testFramesShowMouseover
    end
  end

  -- data for nameplates
  local _, _, icon = GetSpellInfo(self.npBuff[BUFF_INFO.ID])
  self.npBuff[BUFF_INFO.ICON] = icon
end

function TestData:FillNameplates()
  local playerGuid = UnitGUID('player')
  local buffs = {}

  local i = 0
  for k, v in pairs(nameplateMap) do
    if UnitGUID(k) ~= playerGuid then
      i = i + 1

      wipe(buffs)
      for j = 1, (i % 6) + 1 do
        tinsert(buffs, self.npBuff)
      end

      NameplateFrames:Add(v, buffs)
    end
  end
end
